from tracardi_dot_notation.dot_accessor import DotAccessor
from tracardi_plugin_sdk.domain.register import Plugin, Spec, MetaData
from tracardi_plugin_sdk.action_runner import ActionRunner
from tracardi_plugin_sdk.domain.result import Result

from tracardi.process_engine.rules_engine import RulesEngine


class NewEventAction(ActionRunner):

    def __init__(self, **kwargs):
        if 'event' not in kwargs or 'type' not in kwargs['event']:
            raise ValueError("Please define event.type as init parameter.")

        if 'event' not in kwargs or 'properties' not in kwargs['event']:
            raise ValueError("Please define event.properties as init parameter.")

        if kwargs['event']['type'] == 'undefined':
            raise ValueError("Please define event.type in config section. It has default value of undefined.")

        if kwargs['event']['properties'] == 'undefined':
            raise ValueError("Please define event.properties in config section. It has default value of undefined.")

        self.event_type = kwargs['event']['type']
        self.event_properties = kwargs['event']['properties']

    async def run(self, payload):

        dot = DotAccessor(self.profile, self.session, payload, self.event, self.flow)
        try:
            event_properties = dot[self.event_properties]
        except ValueError:
            # If there is error that means properties mus be as defined not a path.
            if isinstance(self.event_properties, dict):
                event_properties = self.event_properties
            else:
                raise ValueError("Properties must be either dict or dot notation path to event, profile or payload. Please correct action configuration.")

        session_copy = self.session.copy(deep=True)
        profile_copy = self.profile.copy(deep=True)

        event_result = await RulesEngine.raise_event(
            self.event_type,
            event_properties,
            session_copy,
            profile_copy,
            source_id=None)

        return Result(port="result", value=event_result)


def register() -> Plugin:
    return Plugin(
        start=False,
        spec=Spec(
            module='tracardi.process_engine.action.new_event_action',
            className='NewEventAction',
            inputs=["payload"],
            outputs=['result'],
            init={
                "event": {"type": "undefined", "properties": "undefined"}
            },
            manual="new_event_action"
        ),
        metadata=MetaData(
            name='New event',
            desc='Raises new event with defined properties. Please define event type and event properties in configuration tab.',
            type='flowNode',
            width=200,
            height=100,
            icon='event'
        )
    )



import asyncio

from tracardi_graph_runner.domain.dag_graph import DagGraph
from tracardi_graph_runner.utils.dag_processor import DagProcessor


data = {
    "nodes": [
        {"id": "n1", "outputs": ["TRUE", "FALSE"], "inputs": [], "className": "task1",
         "module": 'tracardi.tests.manual.plugins.plugins', "init": {"condition": "abc"}},

        {"id": "n2", "outputs": ["NODE2_o"], "inputs": ["NODE2_i"], "className": "task2",
         "module": 'tracardi.tests.manual.plugins.plugins'},

        {"id": "n3", "outputs": ['NODE3_o'], "inputs": ["NODE3_i"], "className": "task3",
         "module": 'tracardi.tests.manual.plugins.plugins'},
        {"id": "n4", "outputs": ["NODE4_o"], "inputs": ["NODE4_2_i", "NODE4_3_i", "NODE4_1_i"], "className": "task4",
         "module": 'tracardi.tests.manual.plugins.plugins'},
        {"id": "n5", "outputs": ["NODE5_i"], "inputs": [], "className": "task5", "module": 'tracardi.tests.manual.plugins.plugins'}

    ],

    'edges': [
        {"id": "e1", "source": {"node_id": "n1", "param": "TRUE"}, "target": {"node_id": "n2", "param": "NODE2_i"}},
        {"id": "e2", "source": {"node_id": "n1", "param": "FALSE"}, "target": {"node_id": "n3", "param": "NODE3_i"}},
        {"id": "e3", "source": {"node_id": "n2", "param": "NODE2_o"},
         "target": {"node_id": "n4", "param": "NODE4_2_i"}},
        {"id": "e4", "source": {"node_id": "n3", "param": "NODE3_o"},
         "target": {"node_id": "n4", "param": "NODE4_3_i"}},
        {"id": "e5", "source": {"node_id": "n1", "param": "TRUE"}, "target": {"node_id": "n4", "param": "NODE4_1_i"}},
        {"id": "e6", "source": {"node_id": "n4", "param": "NODE4_o"}, "target": {"node_id": "n5", "param": "NODE5_i"}},
    ]
}
flow = DagGraph(**data)
dag = DagProcessor(flow)
exec_dag = dag.make_execution_dag(start_node_ids=["n1"])
# print(exec_dag.data)
print()
print(exec_dag.serialize())
print()
# exit()
exec_dag.init()
asyncio.run(exec_dag.run({}))



import asyncio

from tracardi_graph_runner.utils.flow_graph_converter import FlowGraphConverter
from tracardi_graph_runner.utils.dag_processor import DagProcessor

from tracardi.domain.context import Context
from tracardi.domain.entity import Entity
from tracardi.domain.event import Event
from tracardi.domain.payload.tracker_payload import TrackerPayload
from tracardi.domain.profile import Profile
from tracardi.domain.session import Session

flow = {
    "nodes": [
        {
            "id": "start",
            "type": "flowNode",
            "position": {
                "x": 380,
                "y": -260
            },
            "data": {
                "start": True,
                "metadata": {
                    "name": "Start",
                    "type": "flowNode",
                    "desc": "Get payload",
                    "width": 100,
                    "height": 100,
                    "icon": "payload"
                },
                "spec": {
                    "className": "PayloadPlugin",
                    "module": "tracardi.tests.manual.plugins.payload.payload_plugin",
                    "inputs": [],
                    "outputs": [
                        "payload"
                    ]
                }
            }
        },
        {
            "id": "763cbc92-5236-4d30-bff2-a10dbbf8c83f",
            "type": "flowNode",
            "position": {
                "x": 380,
                "y": -60
            },
            "data": {
                "metadata": {
                    "name": "Log",
                    "type": "flowNode",
                    "width": 200,
                    "height": 100,
                    "icon": "debug"
                },
                "spec": {
                    "className": "DebugPlugin",
                    "module": "tracardi.tests.manual.plugins.debug.debug_plugin",
                    "inputs": [
                        "payload"
                    ],
                    "outputs": []
                }
            }
        }
    ],
    "edges": [
        {
            "source": "start",
            "sourceHandle": "payload",
            "target": "763cbc92-5236-4d30-bff2-a10dbbf8c83f",
            "targetHandle": "payload",
            "id": "reactflow__edge-9f95ad82-9d7c-4f61-a439-580ba4c730e3payload-763cbc92-5236-4d30-bff2-a10dbbf8c83fpayload",
            "type": "default"
        }
    ]
}

converter = FlowGraphConverter(flow)
x = converter.convert_to_dag_graph()
print(x)
dag = DagProcessor(x)
exec_dag = dag.make_execution_dag()
# print(exec_dag.data)
print()
print(exec_dag.serialize())
print()
# exit()
event = Event(
    id="event-id",
    source=Entity(id="1"),
    session=Entity(id="2"),
    context=Context(),
    type="xxx"
)
session = Session(id="session-id")
profile = Profile(id="profile-id")
init = exec_dag.init(event, session, profile)
asyncio.run(exec_dag.run({}, "flow-1", "event-1"))

